/**
 * OW2 Util
 * Copyright (C) 2008
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DefaultFileDeployableFactoryManager.java
 * --------------------------------------------------------------------------
 */

package org.ow2.util.plan.deploy.deployable.impl.factory;

import java.util.Collection;
import java.util.HashSet;

import org.ow2.util.archive.api.IFileArchive;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.plan.deploy.deployable.api.FileDeployable;
import org.ow2.util.plan.deploy.deployable.api.factory.FileDeployableException;
import org.ow2.util.plan.deploy.deployable.api.factory.IFileDeployableFactory;
import org.ow2.util.plan.deploy.deployable.api.factory.IFileDeployableFactoryManager;

/**
 * A IFileDeployableFactoryManager that owns a collection of managed factories
 * and tries to build a deployable with them one by one.
 * @author mleduque
 */
public class DefaultFileDeployableFactoryManager implements IFileDeployableFactoryManager {
    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(DefaultFileDeployableFactoryManager.class);

    /**
     * FileDeployableFactory objects managed by the manager.
     */
    private Collection<IFileDeployableFactory> fileDeployableFactories = new HashSet<IFileDeployableFactory>();

    /**
     * {@inheritDoc}
     */
    public FileDeployable<?, ?> getFileDeployable(final IFileArchive fileArchive) throws FileDeployableException {
        FileDeployable<?, ?> fileDeployable = null;
        for (IFileDeployableFactory factory : fileDeployableFactories) {
            try {
                fileDeployable = factory.getFileDeployable(fileArchive);
                return fileDeployable;
            } catch (FileDeployableException e) {
                logger.warn("factory {0} doesn''t support archive {1}", factory, fileArchive, e);
            }
        }
        /* No factory in the manager supports the archive type */
        throw new FileDeployableException();
    }

    /**
     * Adds a factory to the list of managed factories.
     * @param factory the added factory
     */
    public void addFileDeployableFactory(final IFileDeployableFactory factory) {
        fileDeployableFactories.add(factory);
    }

    /**
     * Removes a factory from the list of managed factories.
     * @param factory the removed factory.
     */
    public void removeFileDeployableFactory(final IFileDeployableFactory factory) {
        fileDeployableFactories.remove(factory);
    }
}
